"use strict";
Object.defineProperty(exports, "__esModule", { value: true });
const command_1 = require("@oclif/command");
const URL = require("url");
const fs = require("fs");
const path = require("path");
const proxy = require("koa-proxy");
const mount = require("koa-mount");
const commonFlags = require("../common/flags");
const definition_1 = require("../common/definition");
const koa_1 = require("../common/koa");
const swagger_ui_1 = require("../common/swagger-ui");
class SwaggerUI extends command_1.Command {
    async run() {
        const { args, flags } = this.parse(SwaggerUI);
        const { port, logger, bundle } = flags;
        const definition = definition_1.resolveDefinition(args.definition);
        const app = koa_1.createServer({ logger });
        let proxyPath;
        let documentPath;
        let document;
        const openApiFile = 'openapi.json';
        if (definition) {
            if (definition.match('://') && !flags.server && !flags.proxy) {
                // use remote definition
                documentPath = definition;
            }
            else {
                // parse definition
                document = await definition_1.parseDefinition({ definition, servers: flags.server, proxy: flags.proxy });
                documentPath = `./${openApiFile}`;
            }
        }
        // parse opts for Swagger UI from flags
        const swaggerUIOpts = {
            docExpansion: flags.expand,
            displayOperationId: flags.operationids,
            filter: flags.filter,
            deepLinking: flags.deeplinks,
            withCredentials: flags.withcredentials,
            displayRequestDuration: flags.requestduration,
        };
        if (bundle) {
            // bundle files to directory
            const bundleDir = path.resolve(bundle);
            // create a directory if one does not exist
            if (!fs.existsSync(bundleDir)) {
                fs.mkdirSync(bundleDir);
            }
            // copy dist files
            for (const file of fs.readdirSync(swagger_ui_1.swaggerUIRoot)) {
                const src = path.join(swagger_ui_1.swaggerUIRoot, file);
                const target = path.join(bundleDir, file);
                fs.copyFileSync(src, target);
                this.log(`${target}`);
            }
            // copy openapi definition file
            if (document) {
                const openApiPath = path.join(bundleDir, openApiFile);
                fs.writeFileSync(openApiPath, JSON.stringify(document));
                this.log(`${openApiPath}`);
            }
            // write index.html
            const indexPath = path.join(bundleDir, 'index.html');
            fs.writeFileSync(indexPath, swagger_ui_1.getSwaggerUIIndexHTML(Object.assign({ url: documentPath }, swaggerUIOpts)));
            this.log(`${indexPath}`);
        }
        else {
            if (flags.proxy) {
                // set up a proxy for the api
                let serverURL = null;
                if (document && document.servers && document.servers[0]) {
                    serverURL = document.servers[0].url;
                }
                if (flags.server && typeof flags.server === 'object') {
                    serverURL = flags.server[0];
                }
                if (flags.server && typeof flags.server === 'string') {
                    serverURL = flags.server;
                }
                if (!serverURL) {
                    this.error('Unable to find server URL from definition, please provide a --server parameter');
                }
                const api = URL.parse(serverURL);
                const proxyOpts = {
                    host: URL.format({ protocol: api.protocol, host: api.host, port: api.port }),
                    map: (path) => {
                        if (api.pathname === '/') {
                            return path;
                        }
                        return `${api.pathname}${path}`;
                    },
                    jar: flags.withcredentials,
                };
                proxyPath = '/proxy';
                app.use(mount(proxyPath, proxy(proxyOpts)));
            }
            if (document) {
                // serve the openapi file
                app.use(mount(`/${openApiFile}`, (ctx) => {
                    ctx.body = JSON.stringify(document);
                }));
            }
            // serve swagger ui
            app.use(mount('/', swagger_ui_1.serveSwaggerUI(Object.assign({ url: documentPath }, swaggerUIOpts))));
            // start server
            const { port: portRunning } = await koa_1.startServer({ app, port });
            this.log(`Swagger UI running at http://localhost:${portRunning}`);
            if (document) {
                this.log(`OpenAPI definition at http://localhost:${portRunning}/${openApiFile}`);
            }
            if (proxyPath) {
                this.log(`Proxy running at http://localhost:${portRunning}${proxyPath}`);
            }
            this.log();
        }
    }
}
exports.default = SwaggerUI;
SwaggerUI.description = 'serve or bundle a Swagger UI instance';
SwaggerUI.examples = [
    '$ openapi swagger-ui',
    '$ openapi swagger-ui ./openapi.yml',
    '$ openapi swagger-ui ./openapi.yml --bundle outDir',
];
SwaggerUI.flags = Object.assign(Object.assign(Object.assign(Object.assign(Object.assign({}, commonFlags.help()), commonFlags.serverOpts()), commonFlags.servers()), commonFlags.swaggerUIOpts()), { bundle: command_1.flags.string({
        char: 'B',
        description: 'bundle a static site to directory',
        helpValue: 'outDir',
    }), proxy: command_1.flags.boolean({
        description: 'set up a proxy for the api to avoid CORS issues',
        exclusive: ['bundle'],
    }) });
SwaggerUI.args = [
    {
        name: 'definition',
        description: 'input definition file',
    },
];
